import { EditorView } from '@codemirror/view';
import { Extension } from '@codemirror/state';
import { LanguageSupport } from '@codemirror/language';
import { default as React_2 } from 'react';
import { SQLConfig } from '@codemirror/lang-sql';

declare interface CommonProps {
    /**
     * 编辑器的值
     */
    value?: string;
    /**
     * 编辑器初始化使用的值
     */
    defaultValue?: string;
    /**
     * 编辑器内容发生变化触发的回调函数
     */
    onChange?: (value: unknown) => void;
    /**
     * 针对当前输入添加额外的补全候选
     * @deprecated
     */
    extraCompletions?: string[] | ((word: string) => string[] | Promise<string[]>);
    /**
     * 可以配置的主题
     */
    theme?: "dark" | "light";
    /**
     * 是否可编辑
     */
    editable?: boolean;
    /**
     * 拓展方法
     */
    extensions?: (Extension | LanguageSupport)[];
    /**
     * 默认展示的行数
     * @private
     */
    defaultLines?: number;
    /**
     * 额外的props
     */
    [key: string]: unknown;
}

export declare type IEditor = EditorView;

/**
 * format 方法接受的参数
 */
export declare type IFormatOptions = {
    parser(text: string): string | Promise<string>;
};

declare interface JavascriptProps extends CommonProps {
    language: "javascript";
    langOptions?: {
        jsx?: boolean;
        typescript?: boolean;
    };
}

declare interface JsonProps extends CommonProps {
    language: "json";
    langOptions?: never;
}

export declare const ReactCodemirror: React_2.ForwardRefExoticComponent<Pick<ReactCodemirrorProps | StaticCodemirrorProps, string | number> & React_2.RefAttributes<ReactCodemirrorRefValues>>;

declare type ReactCodemirrorProps = JavascriptProps | JsonProps | SqlProps;

export declare type ReactCodemirrorRefValues = {
    format(configs: IFormatOptions): void;
};

declare interface SqlProps extends CommonProps {
    language: "sql";
    langOptions?: SQLConfig;
}

declare interface StaticCodemirrorProps extends CommonProps {
    editable: false;
    onChange: never;
    defaultValue: never;
    language: ReactCodemirrorProps["language"];
    langOptions: ReactCodemirrorProps["langOptions"];
}

export { }
